import XXModels
import XXClient
import Foundation
import XXMessengerClient

public struct PushExtractor {
  enum Constants {
    static let preImage = "preImage"
    static let appGroup = "group.elixxir.messenger"
    static let notificationData = "notificationData"
  }
  
  public var extractFrom: ([AnyHashable: Any]) -> Result<[Push]?, Error>
}

public extension PushExtractor {
  static let live = PushExtractor { dictionary in
    var environment: MessengerEnvironment = .live()
    environment.ndfEnvironment = .mainnet
    environment.serviceList = .userDefaults(
      key: "preImage",
      userDefaults: UserDefaults(suiteName: "group.elixxir.messenger")!
    )
    let messenger = Messenger.live(environment)
    guard let csv = dictionary[Constants.notificationData] as? String,
          let defaults = UserDefaults(suiteName: Constants.appGroup) else {
      return .success(nil)
    }
    do {
      let reports = try messenger.getNotificationReports(notificationCSV: csv)
      return .success(
        reports
          .filter { $0.forMe }
          .filter { $0.type != .silent }
          .filter { $0.type != .default }
          .map { Push(type: $0.type, source: $0.source) }
      )
    } catch {
      return .failure(error)
    }
  }
}
